@@ -128,6 +128,10 @@ class MintStick:
             self.filesystemlist.connect("changed", self.filesystem_selected)
             self.devicelist.connect("changed", self.device_selected)
 
+            self.label_entry.connect("changed", lambda *_: self.update_format_button())
+            self.filesystemlist.connect("changed", lambda *_: self.update_format_button())
+            self.devicelist.connect("changed", lambda *_: self.update_format_button())
+
             # Filesystemlist
             self.fsmodel = Gtk.ListStore(str, str, int, bool, bool)
             #                     id       label    max-length force-upper-case   force-alpha-numeric
@@ -154,26 +158,33 @@ class MintStick:
             # Default's to fat32
             self.filesystemlist.set_active(0)
             if filesystem_arg is not None:
-                itererator = self.fsmodel.get_iter_first()
-                while itererator is not None:
-                    value = self.fsmodel.get_value(itererator, 0)
+                fs_iter = self.fsmodel.get_iter_first()
+                while fs_iter is not None:
+                    value = self.fsmodel.get_value(fs_iter, 0)
                     if value == filesystem_arg:
-                        self.filesystemlist.set_active_iter(itererator)
-                    itererator = self.fsmodel.iter_next(itererator)
+                        self.filesystemlist.set_active_iter(fs_iter)
+                    fs_iter = self.fsmodel.iter_next(fs_iter)
 
-            self.filesystem_selected(self.filesystemlist)
             self.get_devices()
-
-            if usb_path_arg is not None:
-                itererator = self.devicemodel.get_iter_first()
-                while itererator is not None:
-                    value = self.devicemodel.get_value(itererator, 0)
-                    if usb_path_arg in value:
-                        self.devicelist.set_active_iter(itererator)
-                    itererator = self.devicemodel.iter_next(itererator)
+            self.select_device(usb_path_arg)
 
         self.window.show()
 
+    def select_device(self, usb_path=None):
+        if usb_path is not None:
+            device_iter = self.devicemodel.get_iter_first()
+            while device_iter is not None:
+                value = self.devicemodel.get_value(device_iter, 0)
+                if usb_path in value:
+                    self.devicelist.set_active_iter(device_iter)
+                    return
+                device_iter = self.devicemodel.iter_next(device_iter)
+            return
+
+        device_iter = self.devicemodel.get_iter_first()
+        if device_iter is not None:
+            self.devicelist.set_active_iter(device_iter)
+
     def verify(self, button):
         subprocess.Popen(["mint-iso-verify", self.chooser.get_filename()])
 
@@ -206,7 +217,6 @@ class MintStick:
                         drive_id = drive.get_property('id')
                         if drive_id in detected_drives:
                             continue
-                        detected_drives.append(drive_id)
                         self.print_drive(drive)
                         is_usb = str(drive.get_property('connection-bus')) in ['usb', 'cpio', 'sdio']
                         size = int(drive.get_property('size'))
@@ -236,6 +246,7 @@ class MintStick:
 
                             item = "%s (%s) - %s" % (drive_model, name, size)
 
+                            detected_drives.append(drive_id)
                             self.devicemodel.append([name, item])
 
         self.devicelist.set_model(self.devicemodel)
@@ -247,12 +258,12 @@ class MintStick:
             self.go_button.set_sensitive(True)
 
     def filesystem_selected(self, widget):
-        itererator = self.filesystemlist.get_active_iter()
-        if itererator is not None:
-            self.filesystem = self.fsmodel.get_value(itererator, 0)
+        fs_iter = self.filesystemlist.get_active_iter()
+        if fs_iter is not None:
+            self.filesystem = self.fsmodel.get_value(fs_iter, 0)
             self.activate_devicelist()
 
-            self.label_entry.set_max_length(self.fsmodel.get_value(itererator, 2))
+            self.label_entry.set_max_length(self.fsmodel.get_value(fs_iter, 2))
             self.on_label_entry_text_changed(self, self.label_entry)
 
     def file_selected(self, widget):
@@ -263,13 +274,11 @@ class MintStick:
         else:
             self.verify_button.set_sensitive(False)
 
-    def on_label_entry_text_changed(self, widget, data=None):
-        self.label_entry.handler_block(self.label_entry_changed_id)
+    def fix_label_entry_text(self, should_block_handler):
+        if should_block_handler:
+            self.label_entry.handler_block(self.label_entry_changed_id)
 
         active_iter = self.filesystemlist.get_active_iter()
-        value = self.fsmodel.get_value(active_iter, 0)
-        cursor_pos = self.label_entry.props.cursor_position
-
         text = self.label_entry.get_text()
 
         if self.fsmodel.get_value(active_iter, 3):
@@ -278,6 +287,7 @@ class MintStick:
         if self.fsmodel.get_value(active_iter, 4):
             for char in FORBIDDEN_CHARS:
                 text = text.replace(char, "")
+
         try:
             text = unidecode(text)
         except Exception as e:
@@ -286,13 +296,23 @@ class MintStick:
             text = "USB STICK"
 
         self.label_entry.set_text(text)
-        self.go_button.set_sensitive(len(text) > 0)
 
-        length = self.label_entry.get_buffer().get_length()
-        self.label_entry.select_region(length, -1)
-        self.label_entry.set_position(cursor_pos)
+        if should_block_handler:
+            self.label_entry.handler_unblock(self.label_entry_changed_id)
+
+    def on_label_entry_text_changed(self, widget, data=None):
+        self.fix_label_entry_text(True)
+
+    def update_format_button(self):
+        has_valid_device = self.devicelist.get_active_iter() is not None
+        has_valid_fs = self.filesystemlist.get_active_iter() is not None
+        is_enough_length = self.label_entry.get_buffer().get_length() > 0
+
+        if has_valid_device and has_valid_fs and is_enough_length:
+            self.go_button.set_sensitive(True)
+        else:
+            self.go_button.set_sensitive(False)
 
-        self.label_entry.handler_unblock(self.label_entry_changed_id)
 
     def do_format(self, widget):
         if self.debug:
